@@ -22,15 +22,14 @@
  */
 
 #include "P_Net.h"
-#include "ts/InkErrno.h"
-#include "ts/ink_sock.h"
+#include "tscore/InkErrno.h"
+#include "tscore/ink_sock.h"
 #include "P_SSLNextProtocolAccept.h"
 
 // For Stat Pages
 #include "StatPages.h"
 
-volatile int net_accept_number = 0;
-extern std::vector<NetAccept *> naVec;
+int net_accept_number = 0;
 NetProcessor::AcceptOptions const NetProcessor::DEFAULT_ACCEPT_OPTIONS;
 
 NetProcessor::AcceptOptions &
@@ -52,6 +51,7 @@ NetProcessor::AcceptOptions::reset()
   packet_tos            = 0;
   tfo_queue_length      = 0;
   f_inbound_transparent = false;
+  f_proxy_protocol      = false;
   return *this;
 }
 
@@ -124,11 +124,16 @@ UnixNetProcessor::accept_internal(Continuation *cont, int fd, AcceptOptions cons
     Debug("http_tproxy", "Marked accept server %p on port %d as inbound transparent", na, opt.local_port);
   }
 
+  if (opt.f_proxy_protocol) {
+    Debug("http_tproxy", "Marked accept server %p on port %d for proxy protocol", na, opt.local_port);
+  }
+
   int should_filter_int         = 0;
   na->server.http_accept_filter = false;
   REC_ReadConfigInteger(should_filter_int, "proxy.config.net.defer_accept");
-  if (should_filter_int > 0 && opt.etype == ET_NET)
+  if (should_filter_int > 0 && opt.etype == ET_NET) {
     na->server.http_accept_filter = true;
+  }
 
   SessionAccept *sa = dynamic_cast<SessionAccept *>(cont);
   na->proxyPort     = sa ? sa->proxyPort : nullptr;
@@ -149,11 +154,12 @@ UnixNetProcessor::accept_internal(Continuation *cont, int fd, AcceptOptions cons
           NetAccept *a = na->clone();
           snprintf(thr_name, MAX_THREAD_NAME_LENGTH, "[ACCEPT %d:%d]", i - 1, ats_ip_port_host_order(&accept_ip));
           a->init_accept_loop(thr_name);
-          Debug("iocore_net_accept", "Created accept thread #%d for port %d", i, ats_ip_port_host_order(&accept_ip));
+          Debug("iocore_net_accept_start", "Created accept thread #%d for port %d", i, ats_ip_port_host_order(&accept_ip));
         }
 
         // Start the "template" accept thread last.
-        Debug("iocore_net_accept", "Created accept thread #%d for port %d", accept_threads, ats_ip_port_host_order(&accept_ip));
+        Debug("iocore_net_accept_start", "Created accept thread #%d for port %d", accept_threads,
+              ats_ip_port_host_order(&accept_ip));
         snprintf(thr_name, MAX_THREAD_NAME_LENGTH, "[ACCEPT %d:%d]", accept_threads - 1, ats_ip_port_host_order(&accept_ip));
         na->init_accept_loop(thr_name);
 #if !TS_USE_POSIX_CAP
@@ -172,7 +178,12 @@ UnixNetProcessor::accept_internal(Continuation *cont, int fd, AcceptOptions cons
   } else {
     na->init_accept(nullptr);
   }
-  naVec.push_back(na);
+
+  {
+    SCOPED_MUTEX_LOCK(lock, naVecMutex, this_ethread());
+    naVec.push_back(na);
+  }
+
 #ifdef TCP_DEFER_ACCEPT
   // set tcp defer accept timeout if it is configured, this will not trigger an accept until there is
   // data on the socket ready to be read
@@ -195,16 +206,25 @@ UnixNetProcessor::accept_internal(Continuation *cont, int fd, AcceptOptions cons
   return na->action_.get();
 }
 
+void
+NetProcessor::stop_accept()
+{
+  for (auto &na : naVec) {
+    na->stop_accept();
+  }
+}
+
 Action *
 UnixNetProcessor::connect_re_internal(Continuation *cont, sockaddr const *target, NetVCOptions *opt)
 {
   EThread *t             = cont->mutex->thread_holding;
   UnixNetVConnection *vc = (UnixNetVConnection *)this->allocate_vc(t);
 
-  if (opt)
+  if (opt) {
     vc->options = *opt;
-  else
+  } else {
     opt = &vc->options;
+  }
 
   vc->set_context(NET_VCONNECTION_OUT);
   bool using_socks = (socks_conf_stuff->socks_needed && opt->socks_support != NO_SOCKS
@@ -216,7 +236,7 @@ UnixNetProcessor::connect_re_internal(Continuation *cont, sockaddr const *target
                            */
                           !socks_conf_stuff->ip_map.contains(target))
 #endif
-                        );
+  );
   SocksEntry *socksEntry = nullptr;
 
   vc->id          = net_next_connection_number();
@@ -258,10 +278,11 @@ UnixNetProcessor::connect_re_internal(Continuation *cont, sockaddr const *target
       if (lock2.is_locked()) {
         int ret;
         ret = vc->connectUp(t, NO_FD);
-        if ((using_socks) && (ret == CONNECT_SUCCESS))
+        if ((using_socks) && (ret == CONNECT_SUCCESS)) {
           return &socksEntry->action_;
-        else
+        } else {
           return ACTION_RESULT_DONE;
+        }
       }
     }
   }
@@ -273,8 +294,9 @@ UnixNetProcessor::connect_re_internal(Continuation *cont, sockaddr const *target
   }
   if (using_socks) {
     return &socksEntry->action_;
-  } else
+  } else {
     return result;
+  }
 }
 
 Action *
@@ -309,8 +331,9 @@ struct CheckConnect : public Continuation {
 
     case NET_EVENT_OPEN_FAILED:
       Debug("iocore_net_connect", "connect Net open failed");
-      if (!action_.cancelled)
+      if (!action_.cancelled) {
         action_.continuation->handleEvent(NET_EVENT_OPEN_FAILED, (void *)e);
+      }
       break;
 
     case VC_EVENT_WRITE_READY:
@@ -336,22 +359,26 @@ struct CheckConnect : public Continuation {
         }
       }
       vc->do_io_close();
-      if (!action_.cancelled)
+      if (!action_.cancelled) {
         action_.continuation->handleEvent(NET_EVENT_OPEN_FAILED, (void *)-ENET_CONNECT_FAILED);
+      }
       break;
     case VC_EVENT_INACTIVITY_TIMEOUT:
       Debug("iocore_net_connect", "connect timed out");
       vc->do_io_close();
-      if (!action_.cancelled)
+      if (!action_.cancelled) {
         action_.continuation->handleEvent(NET_EVENT_OPEN_FAILED, (void *)-ENET_CONNECT_TIMEOUT);
+      }
       break;
     default:
       ink_assert(!"unknown connect event");
-      if (!action_.cancelled)
+      if (!action_.cancelled) {
         action_.continuation->handleEvent(NET_EVENT_OPEN_FAILED, (void *)-ENET_CONNECT_FAILED);
+      }
     }
-    if (!recursion)
+    if (!recursion) {
       delete this;
+    }
     return EVENT_DONE;
   }
 
@@ -363,9 +390,9 @@ struct CheckConnect : public Continuation {
     recursion++;
     netProcessor.connect_re(this, target, opt);
     recursion--;
-    if (connect_status != NET_EVENT_OPEN_FAILED)
+    if (connect_status != NET_EVENT_OPEN_FAILED) {
       return &action_;
-    else {
+    } else {
       delete this;
       return ACTION_RESULT_DONE;
     }
@@ -396,23 +423,25 @@ NetProcessor::connect_s(Continuation *cont, sockaddr const *target, int timeout,
 
 struct PollCont;
 
-// This is a little odd, in that the actual threads are created before calling the processor.
-int
-UnixNetProcessor::start(int, size_t)
+// This needs to be called before the ET_NET threads are started.
+void
+UnixNetProcessor::init()
 {
   EventType etype = ET_NET;
 
   netHandler_offset = eventProcessor.allocate(sizeof(NetHandler));
   pollCont_offset   = eventProcessor.allocate(sizeof(PollCont));
 
-  n_netthreads = eventProcessor.n_threads_for_type[etype];
-  netthreads   = eventProcessor.eventthread[etype];
-  for (int i = 0; i < n_netthreads; ++i) {
-    initialize_thread_for_net(netthreads[i]);
-    extern void initialize_thread_for_http_sessions(EThread * thread, int thread_index);
-    initialize_thread_for_http_sessions(netthreads[i], i);
+  if (0 == accept_mss) {
+    REC_ReadConfigInteger(accept_mss, "proxy.config.net.sock_mss_in");
   }
 
+  // NetHandler - do the global configuration initialization and then
+  // schedule per thread start up logic. Global init is done only here.
+  NetHandler::init_for_process();
+  NetHandler::active_thread_types[ET_NET] = true;
+  eventProcessor.schedule_spawn(&initialize_thread_for_net, etype);
+
   RecData d;
   d.rec_int = 0;
   change_net_connections_throttle(nullptr, RECD_INT, d, nullptr);
@@ -431,21 +460,13 @@ UnixNetProcessor::start(int, size_t)
     }
   }
 
-  // commented by vijay -  bug 2489945
-  /*if (use_accept_thread) // 0
-     { NetAccept * na = createNetAccept();
-     SET_CONTINUATION_HANDLER(na,&NetAccept::acceptLoopEvent);
-     accept_thread_event = eventProcessor.spawn_thread(na);
-     if (!accept_thread_event) delete na;
-     } */
-
   /*
    * Stat pages
    */
   extern Action *register_ShowNet(Continuation * c, HTTPHdr * h);
-  if (etype == ET_NET)
+  if (etype == ET_NET) {
     statPagesManager.register_http("net", register_ShowNet);
-  return 1;
+  }
 }
 
 // Virtual function allows creation of an
